/*****************************************************************************
 *
 * cddiag.c - show line parameters for a port.
 *
 * Copyright 2000 Digi International (www.digi.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <string.h>
#include <sys/termios.h>
#include <sys/types.h>
#include <fcntl.h>
#include <linux/serial.h>
#include <sys/errno.h>
#include <linux/fs.h>
#include "fas.h"
#include "ctio.h"
#include "els.h"
#include "cdutils.h"


/*
 *	other protos
 */
int  main(int argc, char **argv);
void usage(void); 
int get_line_info(ct_line *lptr, char *devname);
void show_line_info(ct_line *lp);

#define DEV_PATH "/dev/"


/****************************************************************************
 *
 *	main()
 */
int main(int argc, char **argv) {

  int rc;
  ct_line diagline;

  if (argc < 2) {
    usage();
    exit(1);
  }
  if ((rc=get_line_info(&diagline, argv[1]))) {
   if (rc != ENOMEM)
     fprintf(stderr, "cddiag, device %s error:%s\n", argv[1],strerror(rc));
   else
     fprintf(stderr, "cddiag, device %s hasn't been opened yet\n", argv[1]);
  exit(0);
  }
  
  show_line_info(&diagline);
return 0; 
}




/****************************************************************************
 *	Do the actual work on the ports 
 *
 */

/*
 * get_line_info()
 * add /dev/ to a portname if needed, and try to open the
 * port. Using the file descriptor from the open,
 * copy the line parameters via the ct_line lptr.
 * return 0 on success, or error code on return.
 *
 */
int get_line_info(ct_line *lptr, char *devname)
{
char portname[PATH_MAX];
int diag_fd;
int rc = 0;
unsigned int DUTdev=0;
struct stat DUTstat;
 

 portname[PATH_MAX-1] = 0;
 if (*devname != '/') {
   strcpy(portname, DEV_PATH);
   strncat(portname, devname, PATH_MAX - 5);
 }
 else
   strncpy(portname, devname, PATH_MAX);
 
 if (stat(portname, &DUTstat)) {
   rc = errno;
   fprintf(stderr, "Cannot stat %s:%s\n", devname, strerror(rc));
   return rc;
 }
 DUTdev = (int) DUTstat.st_rdev;
 if (!(S_ISCHR(DUTstat.st_mode))) {
   printf("cddiag: '%s' not a character device\n",portname);
   exit(1);
 }
  
 /* open the diag port */
 diag_fd = open(DEV_DIAG, O_RDWR);
 if (diag_fd < 0) {
   fprintf(stderr, "Couldn't open diagnostic node %s: %s\n", DEV_DIAG, strerror(errno));
   exit(1);
 }
 if (diag_fd > 0) {
 lptr->ct_magic = DUTdev;	/* this is how we get DUT devnum to ioctl. It will break
				 if ct_magic isn't the first element in the struct. */
 if (ioctl(diag_fd, STSLINEDIAG, lptr) < 0)
   rc = errno;
 close(diag_fd);
 }
 else
   rc = errno;
 return rc;
}

void show_line_info(struct ct_line *lp)
{
  char on[] = "on";
  char off[]="off";
  printf("\n************************ Port Info ************************\n"); 
 
  printf("els%2.2x port %x\n", 0, lp->port);

  printf("Current packet size %d, print_max=%d\n", lp->curr_size_fassend, lp->print_max);
  printf("Current baudext %d\n\n", lp->cdbaud_ext); 
  printf("Modem status:\n");
  printf("DTR: %s         RTS: %s\n",
	 ((lp->mdmsigs & TIOCM_DTR) ? on : off),
	 ((lp->mdmsigs & TIOCM_RTS) ? on : off));
  printf("DSR: %s         CTS: %s\n",
	 ((lp->mdmsigs & TIOCM_DSR) ? on : off),
	 ((lp->mdmsigs & TIOCM_CTS) ? on : off));
  printf("DCD: %s\n", ((lp->mdmsigs & TIOCM_CAR) ? on : off));

  printf("\nFlags:\n");
  printf("offline  : %s       enabled  : %s    disabling : %s \n", 
	 (lp->line_state & LINE_OFFLINE) ? "Y" : "N",
         (lp->line_state & LINE_ENABLED) ? "Y" : "N",
         (lp->line_state & LINE_DISABLING) ? "Y" : "N");
  printf("loopback : %s         break  : %s     wait_out : %s\n", 
	 (lp->line_state & LINE_LOOPBACKON) ? "Y" : "N",
	 (lp->line_state & LINE_BREAK) ? "Y" : "N",
         (lp->line_state & LINE_WAIT_OUT) ? "Y" : "N");
  printf("set_modem: %s     set_params : %s    get_modem : %s\n", 
	 (lp->line_state & LINE_SET_MODEM) ? "Y" : "N",
	 (lp->line_state & LINE_SET_PARAMS) ? "Y" : "N",
         (lp->line_state & LINE_GET_MODEM) ? "Y" : "N");
  printf("printer  : %s     istopped   : %s      ipending :%s\n", 
	 (lp->line_state & LINE_PRINTER) ? "Y" : "N",
	 (lp->line_state & LINE_ISTOPPED) ? "Y" : "N",
         (lp->line_state & LINE_IPENDING) ? "Y" : "N");
  printf("obusy :    %s     ostopped   : %s\n", 
	 (lp->line_state & LINE_OBUSY) ? "Y" : "N",
         (lp->line_state & LINE_OSTOPPED) ? "Y" : "N");
  printf("wait_timer %s     wait_iflush: %s\n", 
	 (lp->line_state & LINE_WAIT_TIMER) ? "Y" : "N",
	 (lp->line_state & LINE_WAIT_IFLUSH) ? "Y" : "N");

 /*   disp_termios(lptr); */

  printf("\n");

}


/*
 *  usage()
 *
 *	Display the proper usage and exit
 *
 */
void usage(void) 
{
  fprintf(stderr,  "Usage: cddiag devname\n");
}






